{% load i18n %}

{% if TNNT_TEMPLATE_URLS_OWN_WEBSITES %}
    <li role="separator" class="divider"></li>
    {% for website in TNNT_TEMPLATE_URLS_OWN_WEBSITES %}
        <li><a href="{{ website.url }}"{% if website.new_tab %} target="_blank" rel="noopener noreferer"{% endif %}>{{ website.name }}</a></li>
    {% endfor %}
{% endif %}


{% if TNNT_TEMPLATE_URLS_OTHER_WEBSITES %}
    <li role="separator" class="divider"></li>
    {% for website in TNNT_TEMPLATE_URLS_OTHER_WEBSITES %}
        <li><a href="{{ website.url }}"{% if website.new_tab %} target="_blank" rel="noopener noreferer"{% endif %}>{{ website.name }}</a></li>
    {% endfor %}
{% endif %}
