/* Hide placeholders in form elements when they are being focused
------------------------------------------------------------------------------------- */
@media all {
    /* Chrome <=56, Safari < 10 */
    ::-webkit-input-placeholder {
        opacity: 1;
        -webkit-transition: opacity .5s;
        transition: opacity .5s;
    }

    /* FF 4-18 */
    :-moz-placeholder {
        opacity: 1;
        -moz-transition: opacity .5s;
        transition: opacity .5s;
    }

    /* FF 19-51 */
    ::-moz-placeholder {
        opacity: 1;
        -moz-transition: opacity .5s;
        transition: opacity .5s;
    }

    /* IE 10+ */
    :-ms-input-placeholder {
        opacity: 1;
        -ms-transition: opacity .5s;
        transition: opacity .5s;
    }

    /* Modern Browsers */
    ::placeholder {
        opacity: 1;
        transition: opacity .5s;
    }

    /* Chrome <=56, Safari < 10 */
    *:focus::-webkit-input-placeholder {
        opacity: 0;
    }

    /* FF 4-18 */
    *:focus:-moz-placeholder {
        opacity: 0;
    }

    /* FF 19-50 */
    *:focus::-moz-placeholder {
        opacity: 0;
    }

    /* IE 10+ */
    *:focus:-ms-input-placeholder {
        opacity: 0;
    }

    /* Modern Browsers */
    *:focus::placeholder {
        opacity: 0;
    }
}

/* Defaults
------------------------------------------------------------------------------------- */
:root {
    --background-color-default: rgb(39 43 48);
    --background-darkmode-color-default: rgb(0 0 0);
    --background-color-default-transparency: rgba(39 43 48 / 0.75);
    --background-color-light-transparency: rgba(58 63 68 / 0.5);
    --background-panel-default: rgba(200 200 200 / 0.2);
    --background-panel-header-default: rgba(62 68 76 / 0.6);

    --background-color-success: rgba(98 196 98 / 0.5);
    --background-color-success-hover: rgba(79 189 79 / 0.6);
    --background-color-info: rgba(91 192 222 / 0.5);
    --background-color-info-hover: rgba(70 184 218 / 0.6);
    --background-color-warning: rgba(248 148 6 / 0.5);
    --background-color-warning-hover: rgba(223 133 5 / 0.6);
    --background-color-danger: rgba(238 95 91 / 0.5);
    --background-color-danger-hover: rgba(236 72 68 / 0.6);
    --border-color-success: rgb(85 177 66);
    --border-color-info: rgb(42 199 210);
    --border-color-warning: rgb(208 90 5);
    --border-color-danger: rgb(234 54 79);

    --background-color-navigation-default-hover: rgb(62 68 76);
    --background-color-darkmode-navigation-default-active: rgba(78 154 6 / 0.4);

    --white: rgb(255 255 255);
    --black: rgb(0 0 0);

    --background-body-default: url('https://web-cdn.eveonline.com/aws/eveonline/sso/background.jpg') no-repeat center center fixed;

    --backdrop-filter-default: saturate(1.5) blur(1.5em);

    --font-color-default: rgb(200 200 200);
    --font-family-default: Roboto, "Helvetica Neue", sans-serif;
    --font-family-panel-title: Georgia, Times, serif;
    --font-family-code: 'Fira Code', monospace;
    --font-family-code-font-variation: 'Fira Code VF', monospace;
}
